package org.processmining.indatabasepreprocessing.algorithms;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

import javax.swing.JComponent;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.connections.ConnectionManager;
import org.processmining.indatabasepreprocessing.parameters.DBConnectionParameter;
import org.processmining.mixedparadigm.models.mixedparadigm.DeclarativeArc;
import org.processmining.mixedparadigm.models.mixedparadigm.PetrinetWithAutomata;
import org.processmining.mixedparadigm.models.mixedparadigm.PetrinetWithAutomataImpl;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.AlternatePrecedence;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.AlternateResponse;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.AlternateSuccession;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.ChainPrecedence;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.ChainResponse;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.ChainSuccession;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.CoExistence;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.NotChainSuccession;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.NotCoExistence;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.NotSuccession;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.Precedence;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.RespondedExistence;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.Response;
import org.processmining.mixedparadigm.models.mixedparadigm.constraints.binary.Succession;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.ViewSpecificAttributeMap;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.jgraph.ProMJGraphVisualizer;
import org.processmining.models.jgraph.visualization.ProMJGraphPanel;
import org.processmining.models.semantics.petrinet.Marking;

public class InDBPreprocessingDeclare implements InDBPreprocessingAlgorithm  {
	private UIPluginContext context;
	private DBConnectionParameter parameter;
	private Statement stat;
	private PetrinetWithAutomata petriNet;
	private Marking marking;
	
	private HashMap<String, Integer> occurrence;
	private HashMap<List<String>, Integer> no_following;
	private HashMap<List<String>, Integer> no_preceding;
	private HashMap<List<String>, Integer> no_cooccurrence;
	private HashMap<List<String>, Integer> dfr;
	private HashMap<List<String>, Integer> repetition;
	private HashMap<List<String>, Integer> contrariwise_repetition;	
	
	private HashMap<List<String>, Double> respondedExistence;
	private HashMap<List<String>, Double> response;
	private HashMap<List<String>, Double> alternateResponse;
	private HashMap<List<String>, Double> chainResponse;
	private HashMap<List<String>, Double> precedence;
	private HashMap<List<String>, Double> alternatePrecedence;
	private HashMap<List<String>, Double> chainPrecedence;
	private HashMap<List<String>, Double> coExistence;
	private HashMap<List<String>, Double> notCoExistence;
	private HashMap<List<String>, Double> succession;
	private HashMap<List<String>, Double> notSuccession;
	private HashMap<List<String>, Double> alternateSuccession;
	private HashMap<List<String>, Double> chainSuccession;
	private HashMap<List<String>, Double> notChainSuccession;
	
	private ArrayList<String> activities;
	private HashMap<String, Transition> transitions;
	
	public InDBPreprocessingDeclare(UIPluginContext context, Connection conn, DBConnectionParameter parameter) {
		this.context = context;
		this.parameter = parameter;
		try {
			stat = conn.createStatement();
			stat.setFetchSize(100000);
		} catch (SQLException e) {
			e.printStackTrace();
		}		
		
		this.occurrence = new HashMap<String, Integer>();
		this.no_following = new HashMap<List<String>, Integer>();
		this.no_preceding = new HashMap<List<String>, Integer>();
		this.no_cooccurrence = new HashMap<List<String>, Integer>();
		this.dfr = new HashMap<List<String>, Integer>();
		this.repetition = new HashMap<List<String>, Integer>();
		this.contrariwise_repetition = new HashMap<List<String>, Integer>();		
		
		this.respondedExistence = new HashMap<List<String>, Double>();	
		this.response = new HashMap<List<String>, Double>();	
		this.alternateResponse = new HashMap<List<String>, Double>();	
		this.chainResponse = new HashMap<List<String>, Double>();	
		this.precedence = new HashMap<List<String>, Double>();	
		this.alternatePrecedence = new HashMap<List<String>, Double>();	
		this.chainPrecedence = new HashMap<List<String>, Double>();	
		this.coExistence = new HashMap<List<String>, Double>();	
		this.notCoExistence = new HashMap<List<String>, Double>();	
		this.succession = new HashMap<List<String>, Double>();	
		this.notSuccession = new HashMap<List<String>, Double>();	
		this.alternateSuccession = new HashMap<List<String>, Double>();	
		this.chainSuccession = new HashMap<List<String>, Double>();	
		this.notChainSuccession = new HashMap<List<String>, Double>();	
		
		this.activities = new ArrayList<String>();
		this.transitions = new HashMap<String,Transition>();
		
		this.petriNet = new PetrinetWithAutomataImpl("Petrinet With Automata");
		this.marking = new Marking();
	}

	public JComponent visualize() {
		long start = System.nanoTime();
		
		// retrieve
		long startRetrieval = System.nanoTime();
		retrieveDeclares();
		retrieveActivities();
		System.out.print("Declare," + String.format("%.3f", (System.nanoTime() - startRetrieval) / 1000.0) + ";" );
		
		// mine
		long startMining = System.nanoTime();
		countSupport();
		prune(0.0); // parameter need to have threshold as well 
		System.out.print(String.format("%.3f", (System.nanoTime() - startMining) / 1000.0) + ";" );
		
		// to print the constraint
		/*Collection<DeclarativeArc> c = petriNet.getConstraints();		
		Iterator<DeclarativeArc> it = c.iterator();		
		while(it.hasNext()) {
			System.out.println(it.next());
		}*/
		
		// visualization
		long startVisualization = System.nanoTime();
		
		// borrow from MixedParadigm
		ConnectionManager cm = context.getConnectionManager();
		// Try to see if there is an initial marking available for visualization
		try {
			for (InitialMarkingConnection mnc : cm.getConnections(InitialMarkingConnection.class, context, marking)) {
				marking = mnc.getObjectWithRole(InitialMarkingConnection.MARKING);

			}
		} catch (ConnectionCannotBeObtained e) {
			// no initial marking, no problem
			marking = new Marking();
		}

		//		return PetriNetVisualization.getComponent(context, n, m);

		ViewSpecificAttributeMap map = new ViewSpecificAttributeMap();
		for (Place p : marking) {
			String label = "" + marking.occurrences(p);
			map.putViewSpecific(p, AttributeMap.LABEL, label);
			map.putViewSpecific(p, AttributeMap.TOOLTIP, p.getLabel());
			map.putViewSpecific(p, AttributeMap.SHOWLABEL, !label.equals(""));
		}

		ProMJGraphPanel panel = ProMJGraphVisualizer.instance().visualizeGraph(context, petriNet, map);
		System.out.print(String.format("%.3f", (System.nanoTime() - startVisualization) / 1000.0) + ";" );
		
		System.out.print(String.format("%.3f", (System.nanoTime() - start) / 1000.0) + "\n" );
		
		return panel;
	}
	
	private void retrieveActivities() {
		try {
			ResultSet rs = stat.executeQuery("select act from occurrence");		
			while(rs.next()) {
				activities.add(rs.getString(1));
			}			
		} catch (SQLException e) {
			e.printStackTrace();
		} 		
	}
	
	private void retrieveDeclares() {
		String retrieveOccurrence = "select * from occurrence where freq > 0";
		String retrieveNoFollowing = "select * from no_following where freq > 0";
		String retrieveNoPreceding = "select * from no_preceding where freq > 0";
		String retrieveNoCooccurrence = "select * from no_cooccurrence where freq > 0";
		String retrieveDfr = "select * from dfr where freq > 0";
		String retrieveRepetition = "select * from repetition where freq > 0";
		String retrieveContrariwiseRepetition = "select * from contrariwise_repetition where freq > 0";
		
		try {
			ResultSet rs;
					
			// occurrence
			rs = stat.executeQuery(retrieveOccurrence);
			
			while(rs.next()) {
				String eventclass = rs.getString(1);
				int freq = rs.getInt(2);
				occurrence.put(eventclass, freq);
			}
			
			// no following
			rs = stat.executeQuery(retrieveNoFollowing);
			
			while(rs.next()) {
				String eventclass1 = rs.getString(1);
				String eventclass2 = rs.getString(2);
				int freq = rs.getInt(3);				
				no_following.put(Arrays.asList(eventclass1, eventclass2), freq);
			}
			
			// no preceding
			rs = stat.executeQuery(retrieveNoPreceding);
			
			while(rs.next()) {
				String eventclass1 = rs.getString(1);
				String eventclass2 = rs.getString(2);
				int freq = rs.getInt(3);				
				no_preceding.put(Arrays.asList(eventclass1, eventclass2), freq);
			}
			
			// no cooccurrence
			rs = stat.executeQuery(retrieveNoCooccurrence);
			
			while(rs.next()) {
				String eventclass1 = rs.getString(1);
				String eventclass2 = rs.getString(2);
				int freq = rs.getInt(3);				
				no_cooccurrence.put(Arrays.asList(eventclass1, eventclass2), freq);
			}
			
			// dfr
			rs = stat.executeQuery(retrieveDfr);
			
			while(rs.next()) {
				String eventclass1 = rs.getString(1);
				String eventclass2 = rs.getString(2);
				int freq = rs.getInt(3);				
				dfr.put(Arrays.asList(eventclass1, eventclass2), freq);
			}
			
			// repetition
			rs = stat.executeQuery(retrieveRepetition);
			
			while(rs.next()) {
				String eventclass1 = rs.getString(1);
				String eventclass2 = rs.getString(2);
				int freq = rs.getInt(3);				
				repetition.put(Arrays.asList(eventclass1, eventclass2), freq);
			}
			
			// contrariwise repetition
			rs = stat.executeQuery(retrieveContrariwiseRepetition);
			
			while(rs.next()) {
				String eventclass1 = rs.getString(1);
				String eventclass2 = rs.getString(1);
				int freq = rs.getInt(3);				
				contrariwise_repetition.put(Arrays.asList(eventclass1, eventclass2), freq);
			}
			
		} catch (SQLException e) {
			e.printStackTrace();
		} 
	}
	
	private void countSupport() {
		for(int i = 0; i < activities.size(); i++) {
			String a = activities.get(i);
			for(int j = 0; j < activities.size(); j++) {
				if(i != j) {
					String b = activities.get(j);
					
					List<String> sourceTarget = Arrays.asList(a,b);
					List<String> targetSource = Arrays.asList(b,a);			
				
					Integer dec2 = (occurrence.get(a) == null) ? 0 : occurrence.get(a);
					Integer dec2Rev = (occurrence.get(b) == null) ? 0 : occurrence.get(b);
					Integer sumDec2 = dec2 + dec2Rev;
					Integer dec3 = (no_following.get(sourceTarget) == null) ? 0 : no_following.get(sourceTarget);
					Integer dec4 = (no_preceding.get(sourceTarget) == null) ? 0 : no_preceding.get(sourceTarget);
					Integer dec4Rev = (no_preceding.get(targetSource) == null) ? 0 : no_preceding.get(targetSource);
					Integer dec5 = (no_cooccurrence.get(sourceTarget) == null) ? 0 : no_cooccurrence.get(sourceTarget);
					Integer dec5Rev = (no_cooccurrence.get(targetSource) == null) ? 0 : no_cooccurrence.get(targetSource);
					Integer dec6 = (dfr.get(sourceTarget) == null) ? 0 : dfr.get(sourceTarget);
					Integer dec7 = (dfr.get(targetSource) == null) ? 0 : dfr.get(targetSource);
					Integer dec7Rev = dec6;
					Integer dec8 = (repetition.get(sourceTarget) == null) ? 0 : repetition.get(sourceTarget);
					Integer dec9 = (contrariwise_repetition.get(sourceTarget) == null) ? 0 : contrariwise_repetition.get(sourceTarget);
					Integer dec9Rev = (contrariwise_repetition.get(targetSource) == null) ? 0 : contrariwise_repetition.get(targetSource);
					
					if(dec2 != 0) {
						Double re = 1.0 - ((double) dec5/dec2);
						if(re>0.0) respondedExistence.put(sourceTarget, re);		
						
						Double r = 1.0 - ((double) dec3/dec2);
						if(r>0.0) response.put(sourceTarget, r);
						
						Double ar = 1.0 - ((double) (dec3+dec8)/dec2);
						if(ar>0.0) alternateResponse.put(sourceTarget, ar);
						
						Double cr = (double) (dec6/dec2);
						if(cr>0.0) chainResponse.put(sourceTarget, cr);
						
						Double p = 1.0 - ((double) dec4/dec2);
						if(p>0.0) precedence.put(targetSource, p);
						
						Double ap = 1.0 - ((double) (dec4+dec9)/dec2);
						if(ap>0.0) alternatePrecedence.put(targetSource, ap);
						
						Double cp = (double) dec7/dec2;
						if(cp>0.0) chainPrecedence.put(targetSource, cp);
					}
					
					if(sumDec2 != 0) {
						Double nce = (double) (dec5 + dec5Rev)/sumDec2;
						Double ce = 1.0 - nce;
						if(ce>0.0) coExistence.put(sourceTarget, ce);
						if(nce>0.0) notCoExistence.put(sourceTarget, nce);
						
						Double ns = (double) (dec3 + dec4Rev)/sumDec2;
						Double s = 1.0 - ns;
						if(s>0.0) succession.put(sourceTarget, s);
						if(ns>0.0) notSuccession.put(sourceTarget, ns);
						
						Double as = 1.0 - ((double) (dec8 + dec3 + dec9Rev + dec4Rev)/sumDec2);
						if(as>0.0) alternateSuccession.put(sourceTarget, as);
						
						 Double cs = (double) (dec6 + dec7Rev)/sumDec2;
						 Double ncs = 1.0 - cs;
						 if(cs>0.0) chainSuccession.put(sourceTarget, cs);
						 if(ncs>0.0) notChainSuccession.put(sourceTarget, ncs);
					} 
				}
			}
		}
	}
	
	private void prune(double threshold) {		
		
		for(int i = 0; i < activities.size(); i++) {
			String act = activities.get(i);
			Transition t = petriNet.addTransition(act);
			transitions.put(act, t);
		}
		
		for(int i = 0; i < activities.size(); i++) {
			String a = activities.get(i);
			for(int j = 0; j < activities.size(); j++) {
				String b = activities.get(j);
				
				List<String> sourceTarget = Arrays.asList(a,b);
				List<String> targetSource = Arrays.asList(b,a);					
				
				Double re = (respondedExistence.get(sourceTarget) == null) ? 0.0 : respondedExistence.get(sourceTarget);			
				Double r = (response.get(sourceTarget) == null) ? 0.0 : response.get(sourceTarget);		
				Double ar = (alternateResponse.get(sourceTarget) == null) ? 0.0 : alternateResponse.get(sourceTarget);		
				Double cr = (chainResponse.get(sourceTarget) == null) ? 0.0 : chainResponse.get(sourceTarget);		
				Double pR = (precedence.get(targetSource) == null) ? 0.0 : precedence.get(targetSource);		
				Double apR = (alternatePrecedence.get(targetSource) == null) ? 0.0 : alternatePrecedence.get(targetSource);		
				Double cpR = (chainPrecedence.get(targetSource) == null) ? 0.0 : chainPrecedence.get(targetSource);		
				
				boolean existRespondedExistence = false;
				
				// chain response
				if(cr.equals(ar) && cr >= threshold) {
					existRespondedExistence = true;
					petriNet.addConstraint(transitions.get(a), transitions.get(b), new ChainResponse());
				// alternate response
				} else if(ar.equals(r) && ar >= threshold) {
					existRespondedExistence = true;
					petriNet.addConstraint(transitions.get(a), transitions.get(b), new AlternateResponse());
				// response
				} else if(r.equals(re) && r >= threshold) {
					existRespondedExistence = true;
					petriNet.addConstraint(transitions.get(a), transitions.get(b), new Response());
				}
				
				// chain precedence
				if(cpR != null && cpR.equals(apR) && cpR >= threshold) {
					existRespondedExistence = true;
					petriNet.addConstraint(transitions.get(b), transitions.get(a), new ChainPrecedence());
				// alternate precedence
				} else if(apR != null && apR.equals(pR) && apR >= threshold) {
					existRespondedExistence = true;
					petriNet.addConstraint(transitions.get(b), transitions.get(a), new AlternatePrecedence());
				// precedence
				} else if(pR != null && pR.equals(re) && pR >= threshold) {
					existRespondedExistence = true;
					petriNet.addConstraint(transitions.get(b), transitions.get(a), new Precedence());
				} 
				
				// responded existence
				if(re != null && !existRespondedExistence && re >= threshold) {
					petriNet.addConstraint(transitions.get(a), transitions.get(b), new RespondedExistence());
				}
			} 
		}  
	
		 // MUTUAL RELATION		
		for(int i = 0; i < activities.size(); i++) {
			String a = activities.get(i);
			for(int j = 0; j < activities.size(); j++) {
				String b = activities.get(j);
				
				List<String> sourceTarget = Arrays.asList(a,b);
				List<String> targetSource = Arrays.asList(b,a);	
				
				Double re = (respondedExistence.get(sourceTarget) == null) ? 0.0 : respondedExistence.get(sourceTarget);			
				Double r = (response.get(sourceTarget) == null) ? 0.0 : response.get(sourceTarget);	
				Double ar = (alternateResponse.get(sourceTarget) == null) ? 0.0 : alternateResponse.get(sourceTarget);	
				Double cr = (chainResponse.get(sourceTarget) == null) ? 0.0 : chainResponse.get(sourceTarget);	
				Double p = (precedence.get(sourceTarget) == null) ? 0.0 : precedence.get(sourceTarget);	
				Double ap = (alternatePrecedence.get(sourceTarget) == null) ? 0.0 : alternatePrecedence.get(sourceTarget);	
				Double cp = (chainPrecedence.get(sourceTarget) == null) ? 0.0 : chainPrecedence.get(sourceTarget);	
				Double ce = (coExistence.get(sourceTarget) == null) ? 0.0 : coExistence.get(sourceTarget);	
				Double reR = (respondedExistence.get(targetSource) == null) ? 0.0 : respondedExistence.get(targetSource);	
				Double s = (succession.get(sourceTarget) == null) ? 0.0 : succession.get(sourceTarget);	
				Double as = (alternateSuccession.get(sourceTarget) == null) ? 0.0 : alternateSuccession.get(sourceTarget);	
				Double cs = (chainSuccession.get(sourceTarget) == null) ? 0.0 : chainSuccession.get(sourceTarget);	
				
				// chain succession
				if(cs.equals(cr) && cs.equals(cp) && cs >= threshold) {
					DeclarativeArc arc1 = new DeclarativeArc(transitions.get(a), transitions.get(b), new ChainResponse());
					DeclarativeArc arc2 = new DeclarativeArc(transitions.get(a), transitions.get(b), new ChainPrecedence());
					petriNet.removeConstraint(arc1);
					petriNet.removeConstraint(arc2);
					petriNet.addConstraint(transitions.get(a), transitions.get(b), new ChainSuccession());
				// alternate succession
				} else if(as.equals(ar) && as.equals(ap) && as >= threshold) {
					DeclarativeArc arc1 = new DeclarativeArc(transitions.get(a), transitions.get(b), new AlternateResponse());
					DeclarativeArc arc2 = new DeclarativeArc(transitions.get(a), transitions.get(b), new AlternatePrecedence());
					petriNet.removeConstraint(arc1);
					petriNet.removeConstraint(arc2);
					petriNet.addConstraint(transitions.get(a), transitions.get(b), new AlternateSuccession());
				// succession
				} else if(s.equals(r) && s.equals(p) && s >= threshold) {
					DeclarativeArc arc1 = new DeclarativeArc(transitions.get(a), transitions.get(b), new Response());
					DeclarativeArc arc2 = new DeclarativeArc(transitions.get(a), transitions.get(b), new Precedence());
					petriNet.removeConstraint(arc1);
					petriNet.removeConstraint(arc2);
					petriNet.addConstraint(transitions.get(a), transitions.get(b), new Succession());
				// coExistence
				} else if(ce.equals(re) && ce.equals(reR) && ce >= threshold) {
					DeclarativeArc arc1 = new DeclarativeArc(transitions.get(a), transitions.get(b), new RespondedExistence());
					DeclarativeArc arc2 = new DeclarativeArc(transitions.get(b), transitions.get(a), new RespondedExistence());
					petriNet.removeConstraint(arc1);
					petriNet.removeConstraint(arc2);
					petriNet.addConstraint(transitions.get(a), transitions.get(b), new CoExistence());
				}
			}
		}
		
		// NEGATIVE RELATION
		for(int i = 0; i < activities.size(); i++) {
			String a = activities.get(i);
			for(int j = 0; j < activities.size(); j++) {
				String b = activities.get(j);
				
				List<String> sourceTarget = Arrays.asList(a,b);			
		
				Double ce = (coExistence.get(sourceTarget) == null) ? 0.0 : coExistence.get(sourceTarget);
				Double s = (succession.get(sourceTarget) == null) ? 0.0 : succession.get(sourceTarget);
				Double cs = (chainSuccession.get(sourceTarget) == null) ? 0.0 : chainSuccession.get(sourceTarget);
				Double ncs = (notChainSuccession.get(sourceTarget) == null) ? 0.0 : notChainSuccession.get(sourceTarget);
				Double ns = (notSuccession.get(sourceTarget) == null) ? 0.0 : notSuccession.get(sourceTarget);
				Double nce = (notCoExistence.get(sourceTarget) == null) ? 0.0 : notCoExistence.get(sourceTarget);
				
				// not co existence
				if(ce <= nce && nce >= threshold) {
					petriNet.addConstraint(transitions.get(a), transitions.get(b), new NotCoExistence());
				// not succession
				} else if(s <= ns && ns >= threshold) {
					petriNet.addConstraint(transitions.get(a), transitions.get(b), new NotSuccession());
				 //  not chain succession
				} else if(cs <= ncs && ncs >= threshold) {
					petriNet.addConstraint(transitions.get(a), transitions.get(b), new NotChainSuccession());
				}
			}		
		}
	}
}
