package org.processmining.indatabasepreprocessing.algorithms;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashMap;

import javax.swing.JComponent;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.indatabasepreprocessing.parameters.DBConnectionParameter;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.plugins.InductiveMiner.dfgOnly.Dfg;
import org.processmining.plugins.InductiveMiner.dfgOnly.DfgImpl;
import org.processmining.plugins.InductiveMiner.dfgOnly.DfgMiningParameters;
import org.processmining.plugins.InductiveMiner.dfgOnly.DfgMiningParametersIMd;
import org.processmining.plugins.InductiveMiner.dfgOnly.plugins.IMdProcessTree;
import org.processmining.plugins.petrinet.PetriNetVisualization;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.conversion.ProcessTree2Petrinet;
import org.processmining.processtree.conversion.ProcessTree2Petrinet.InvalidProcessTreeException;
import org.processmining.processtree.conversion.ProcessTree2Petrinet.NotYetImplementedException;
import org.processmining.processtree.conversion.ProcessTree2Petrinet.PetrinetWithMarkings;

@SuppressWarnings("deprecation")
public class InDBPreprocessingInductiveMiner implements InDBPreprocessingAlgorithm {

	private UIPluginContext context;
	private DBConnectionParameter parameter;
	private Statement stat;
	private Petrinet petriNet;
	private Dfg dfg;
	
	public InDBPreprocessingInductiveMiner(UIPluginContext context, Connection conn, DBConnectionParameter parameter) {
		this.context = context;
		this.parameter = parameter;
		try {
			stat = conn.createStatement();
			stat.setFetchSize(100000);
		} catch (SQLException e) {
			e.printStackTrace();
		}		
	}

	public JComponent visualize() {
		long start = System.nanoTime();
		
		// get DFG
		long startRetrieve = System.nanoTime();
		try {
			getDfg();
		} catch (SQLException e) {
			e.printStackTrace();
		}
		System.out.print("IM," + String.format("%.3f", (System.nanoTime() - startRetrieve) / 1000.0) + ";" );
		
		// mine
		long startMining = System.nanoTime();
		try {
			mine();
		} catch (NotYetImplementedException | InvalidProcessTreeException e) {
			e.printStackTrace();
		}	
		System.out.print(String.format("%.3f", (System.nanoTime() - startMining) / 1000.0) + ";" );
		
		// visualization
		long startVisualization = System.nanoTime();
		PetriNetVisualization pnVis = new PetriNetVisualization();
		JComponent component = pnVis.visualize(context, petriNet);		
		System.out.print(String.format("%.3f", (System.nanoTime() - startVisualization) / 1000.0) + ";" );
		
		System.out.print(String.format("%.3f", (System.nanoTime() - start) / 1000.0) + "\n" );
		
		return component;
	}
	
	private void mine() throws NotYetImplementedException, InvalidProcessTreeException {
		// mine process tree
		DfgMiningParameters param = new DfgMiningParametersIMd();
		ProcessTree processtree = IMdProcessTree.mineProcessTree(dfg, param);
		
		// convert to petri net
		ProcessTree2Petrinet pt2pn = new ProcessTree2Petrinet();
		PetrinetWithMarkings result = pt2pn.convert(processtree, false);
		
		petriNet = result.petrinet;
	}
	
	private void getDfg() throws SQLException {
		dfg = new DfgImpl();
		HashMap<String,Integer> eventClassIndex = new HashMap<String, Integer>();
		int index = 0;
		
		// PAIR of activities
		ResultSet rs = stat.executeQuery("select * from dfr");
		
		while(rs.next()) {
			String elem1 = rs.getString(1);
			String elem2 = rs.getString(2);
			int freq = rs.getInt(3);
									
			if(!eventClassIndex.containsKey(elem1)) {
				eventClassIndex.put(elem1, index);
				index++;
			}				
			if(!eventClassIndex.containsKey(elem2)) {
				eventClassIndex.put(elem2, index);
				index++;
			}
			
			XEventClass ec1 = new XEventClass(elem1, eventClassIndex.get(elem1));
			XEventClass ec2 = new XEventClass(elem2, eventClassIndex.get(elem2));					
			
			dfg.addActivity(ec1);
			dfg.addActivity(ec2);
			dfg.addDirectlyFollowsEdge(ec1, ec2, freq);			
		}
		
		//START activities
		rs = stat.executeQuery("select act, count(*) from start_event group by act");
		
		while(rs.next()) {
			String eventLabel = rs.getString(1);
			int freq = rs.getInt(2);
							
			if(!eventClassIndex.containsKey(eventLabel)) {
				eventClassIndex.put(eventLabel, index);
				index++;
			}				
			
			XEventClass startEventLabel = new XEventClass(eventLabel, eventClassIndex.get(eventLabel));	
			
			dfg.addActivity(startEventLabel);
			dfg.addStartActivity(startEventLabel, freq);
		}
		
		//LAST activities
		rs = stat.executeQuery("select act, count(*) from last_event group by act");
		
		while(rs.next()) {
			String eventLabel = rs.getString(1);
			int freq = rs.getInt(2);
							
			if(!eventClassIndex.containsKey(eventLabel)) {
				eventClassIndex.put(eventLabel, index);
				index++;
			}				
			
			XEventClass lastEventLabel = new XEventClass(eventLabel, eventClassIndex.get(eventLabel));	
			
			dfg.addActivity(lastEventLabel);
			dfg.addEndActivity(lastEventLabel, freq);
		}
	}
}
