package org.processmining.indatabasepreprocessing.plugins;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginCategory;
import org.processmining.framework.plugin.annotations.PluginLevel;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.indatabasepreprocessing.algorithms.InDBPreprocessingController;
import org.processmining.indatabasepreprocessing.parameters.DBConnectionParameter;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;

@Plugin(name = "Mine with IncPM", level = PluginLevel.PeerReviewed, categories = { PluginCategory.Analytics }, parameterLabels = { "IncPM Database Controller", "Petri Net" }, 
returnLabels = { "IncPM Results" }, returnTypes = { InDBPreprocessingController.class }, userAccessible = true, help = InDBPreprocessingHelp.TEXT)
public class InDBPreprocessingPlugin {
	
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "A. Syamsiyah", email = "a.syamsiyah@tue.nl", pack = "InDatabasePreprocessing")
	@PluginVariant(variantLabel = "Mining with In Database Preprocessing", requiredParameterLabels = { 0 })
	public InDBPreprocessingController main(UIPluginContext context, DBConnectionParameter parameter) {
		
		InDBPreprocessingController controller = new InDBPreprocessingController(context, parameter);
		return controller;
	}
	
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "A. Syamsiyah", email = "a.syamsiyah@tue.nl", pack = "InDatabasePreprocessing")
	@PluginVariant(variantLabel = "Mine with IncPM", requiredParameterLabels = { 0, 1 })
	public InDBPreprocessingController main(UIPluginContext context, DBConnectionParameter parameter, Petrinet net) {
		
		InDBPreprocessingController controller = new InDBPreprocessingController(context, parameter, net);
		return controller;
	}

}
